::cisco::eem::event_register_none maxrun 60
#-
# Copyright (c) 2010 Joe Marcus Clarke <jclarke@cisco.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#
# TECHNICAL ASSISTANCE CENTER (TAC) SUPPORT IS NOT AVAILABLE FOR THIS SCRIPT.
# For questions or help, send email to ask-easy@cisco.com.
#
# This policy tests various network management notification protocols.
# Currently, syslog, SNMP trap, email, and Smart Call Home tests can be
# performed. This script can be controlled either by environment variables
# or command line options.
#
# Command line options are only available if the device supports EEM 2.4 or
# higher. If command line options are specified, environment variables will
# not be used.  To get help on usage with command line options, run the
# following command:
#
# event manager run no_easy_nmstest.tcl --help
#
# This policy supports the following environment variables:
#
# Syslog options:
#
# easy_nmstest_syslog      : Set the value to 1 to perform the syslog
#                            test.
# easy_nmstest_syslog_fac  : Custom syslog facility to use for the
#                            syslog test.  The default is
#                            HA_EM.  This variable
#                            is only affective when running EEM 3.1
#                            or higher, or if the device supports the
#                            'syslog:' file system.
# easy_nmstest_syslog_sevs : A comma-separated list of syslog
#                            severities to test.  Each severity
#                            should be an integer between 0 and 7
#                            inclusive.  If omitted, all syslog
#                            severities will be tested.
#
#
# SNMP Trap options:
#
# easy_nmstest_trap        : Set the value to 1 to perform the SNMP
#                            trap test.
#
#
# Email options:
#
# easy_nmstest_email       : Set the value to 1 to perform the email
#                            test.
# easy_nmstest_email_to    : Comma-separated list of addresses to
#                            which the test email will be sent.
#                            If not set, then the _email_to
#                            variable will be used instead.
# easy_nmstest_email_from  : Email address from which the test email
#                            will be sent.  If not set, then the
#                            _email_from variable will be used
#                            instead.
# easy_nmstest_email_cc    : (optional) Comma-separated list of
#                            addresses to which the test email will
#                            be carbon-copied.  If not set, then
#                            _email_cc variable will bve used
#                            instead.
# easy_nmstest_email_server: SMTP server which will relay the test
#                            email.  If not set, then the _email_server
#                            variable will be used instead.
#
# If the device supports EEM 3.0 or higher, then the following two
# environment variables can be specified:
#
# easy_nmstest_email_source_addr : (optional) Source IP address to use
#                                  when sending the test email.  If
#                                  omitted, the email will be sent from
#                                  the interface closest to the SMTP server.
#                                  If not set, then the
#                                  _email_source_addr variable
#                                  will be used instead.
# easy_nmstest_source_intf       : (optional) Source interface to use
#                                  when sending the test email.  If
#                                  omitted, the email will be sent from
#                                  the interface closest to the SMTP server.
#                                  If not set, then the
#                                  _email_source_intf variable will
#                                  be used instead.
#
# If the device supports EEM 2.4 or higher, then the following two
# environment variables can be specified:
#
# easy_nmstest_email_username    : (optional) SMTP server username.  If
#                                  omitted, unauthenticated SMTP is
#                                  assumed.  If not set, then the
#                                  _email_username variable will be
#                                  used instead.
# easy_nmstest_email_password    : (optional) SMTP server password.  This
#                                  can only be set if _email_username is
#                                  also set.  If not set, then the
#                                  _email_password variable will
#                                  be used instead.
#
#
# Smart Call Home options:
#
# easy_nmstest_callhome         : Set the value to 1 to perform the Smart Call
#                                 Home test.
# easy_nmstest_callhome_profile : Set to the name of the Smart Call Home
#                                 profile to test.
#
# $Id: no_easy_nmstest.tcl,v 1.12 2010/02/24 02:35:46 marcus Exp $
#

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

proc run_cli { clist } {
    set rbuf ""

    if {[llength $clist] < 1} {
	return -code ok $rbuf
    }

    if {[catch {cli_open} result]} {
        return -code error $result
    } else {
	array set cliarr $result
    }

    if {[catch {cli_exec $cliarr(fd) "enable"} result]} {
        return -code error $result
    }

    if {[catch {cli_exec $cliarr(fd) "term length 0"} result]} {
        return -code error $result
    }

    foreach cmd $clist {
	if {[catch {cli_exec $cliarr(fd) $cmd} result]} {
            return -code error $result
	}

	append rbuf $result
    }

    if {[catch {cli_close $cliarr(fd) $cliarr(tty_id)} result]} {
        puts "WARNING: $result"
    }

    return -code ok $rbuf
}

proc usage { } {
    global has_eem_2_4
    global policy_name

    if { ! $has_eem_2_4 } {
	puts "Prior to EEM 2.4, $policy_name can only be controlled via"
	puts "environment variables."
	puts ""
	cmdhelp
    } else {
	puts "$policy_name can be controlled using environment variable"
	puts "or command line options."
	puts ""
	puts "If command line options are specified, environment variables"
	puts "will not be used."

        puts ""
        puts "Usage: event manager run $policy_name"
        puts ""
        puts ""
        puts "Usage: event manager run $policy_name --tests <tests> \[--facility <facility>]"
        puts {       [--severities <severities>] [--from <from address>]}
        puts {       [--to <to address(es)>] [--cc <cc address(es)]}
        puts {       [--server <SMTP server>] [--source <source intf or IP>]}
        puts {       [--username <username>] [--password <password>]}
        puts ""
        puts ""
        puts "Usage: event manager run $policy_name --help"
    }
}

proc cmdhelp { } {
    global has_eem_2_4
    global has_eem_3_0
    global policy_name
    global DEF_SYSLOG_FACILITY

    puts {This policy tests various network management notification}
    puts {protocols.  Currently, syslog, SNMP trap, email, and Smart Call}
    puts {Home tests can be performed.  This script can be controlled either}
    puts {by environment variables or command line options.}
    puts ""
    if { $has_eem_2_4 } {
	puts "$policy_name can be controlled using environment variable"
	puts "or command line options."
	puts ""
	puts "If command line options are specified, environment variables"
	puts "will not be used."
    }

    puts ""

    puts {Environment Variables:}
    puts ""
    puts ""
    puts "Usage: event manager run $policy_name"
    puts ""
    puts {Syslog options:}
    puts ""
    puts {easy_nmstest_syslog      : Set the value to 1 to perform the syslog}
    puts {                           test.}
    puts {easy_nmstest_syslog_fac  : Custom syslog facility to use for the}
    puts {                           syslog test.  The default is}
    puts "                           $DEF_SYSLOG_FACILITY.  This variable"
    puts {                           is only affective when running EEM 3.1}
    puts {                           or higher, or if the device supports the}
    puts {                           'syslog:' file system.}
    puts {easy_nmstest_syslog_sevs : A comma-separated list of syslog}
    puts {                           severities to test.  Each severity}
    puts {                           should be an integer between 0 and 7}
    puts {                           inclusive.  If omitted, all syslog}
    puts {                           severities will be tested.}
    puts ""
    puts ""
    puts {SNMP Trap options:}
    puts ""
    puts {easy_nmstest_trap        : Set the value to 1 to perform the SNMP}
    puts {                           trap test.}
    puts ""
    puts ""
    puts {Email options:}
    puts ""
    puts {easy_nmstest_email       : Set the value to 1 to perform the email}
    puts {                           test.}
    puts {easy_nmstest_email_to    : Comma-separated list of addresses to}
    puts {                           which the test email will be sent.}
    puts {                           If not set, then the _email_to}
    puts {                           variable will be used instead.}
    puts {easy_nmstest_email_from  : Email address from which the test email}
    puts {                           will be sent.  If not set, then the}
    puts {                           _email_from variable will be used}
    puts {                           instead.}
    puts {easy_nmstest_email_cc    : (optional) Comma-separated list of}
    puts {                           addresses to which the test email will}
    puts {                           be carbon-copied.  If not set, then}
    puts {                           _email_cc variable will bve used}
    puts {                           instead.}
    puts {easy_nmstest_email_server: SMTP server which will relay the test}
    puts {                           email.  If not set, then the _email_server}
    puts {                           variable will be used instead.}
    if { $has_eem_3_0 } {
        puts {easy_nmstest_email_source_addr : (optional) Source IP address to use}
        puts {                                 when sending the test email.  If}
        puts {                                 omitted, the email will be sent from}
        puts {                                 the interface closest to the SMTP server.}
	puts {                                 If not set, then the}
	puts {                                 _email_source_addr variable}
	puts {                                 will be used instead.}
        puts {easy_nmstest_source_intf       : (optional) Source interface to use}
        puts {                                 when sending the test email.  If}
        puts {                                 omitted, the email will be sent from}
        puts {                                 the interface closest to the SMTP server.}
	puts {                                 If not set, then the}
	puts {                                 _email_source_intf variable will}
	puts {                                 be used instead.}
    }
    if { $has_eem_2_4 } {
        puts {easy_nmstest_email_username    : (optional) SMTP server username.  If}
        puts {                                 omitted, unauthenticated SMTP is}
        puts {                                 assumed.  If not set, then the}
	puts {                                 _email_username variable will be}
	puts {                                 used instead.}
        puts {easy_nmstest_email_password    : (optional) SMTP server password.  This}
        puts {                                  can only be set if _email_username is}
        puts {                                  also set.  If not set, then the}
	puts {                                  _email_password variable will}
	puts {                                  be used instead.}
    }

    puts ""
    puts ""
    puts {Smart Call Home Options:}
    puts ""
    puts {easy_nmstest_callhome            : Set to 1 to perform the Smart Call}
    puts {                                   Home test.}
    puts {easy_nmstest_callhome_profile    : Set to the name of the Smart Call}
    puts {                                   Home profile to test.}

    if { $has_eem_2_4 } {
	puts {Command Line Options:}
	puts ""
	puts ""
        puts "Usage: event manager run $policy_name \[OPTIONS...]"
        puts { --tests              A comma-separated lists of tests to perform.}
        puts {                      Currently the available tests are 'syslog',}
        puts {                      'trap', 'email', and 'callhome'.}
        puts { --facility           Custom syslog facility to use for the}
        puts {                      syslog test.  The default is}
        puts "                      $DEF_SYSLOG_FACILITY.  This variable"
        puts {                      is only affective when running EEM 3.1}
        puts {                      or higher, or if the device supports the}
        puts {                      'syslog:' file system.}
        puts { --severities         A comma-separated list of syslog}
        puts {                      severities to test.  Each severity}
        puts {                      should be an integer between 0 and 7}
        puts {                      inclusive.  If omitted, all syslog}
        puts {                      severities will be tested.}
	puts { --profile            The name of the Smart Call Home profile}
	puts {                      to be tested.}
        puts { --to                 Comma-separated list of addresses to}
        puts {                      which the test email will be sent.}
        puts { --from               Email address from which the test email}
        puts {                      will be sent.}
        puts { --cc                 (optional) Comma-separated list of}
        puts {                      addresses to which the test email will}
        puts {                      be carbon-copied.}
        puts { --server             SMTP server which will relay the test}
        puts {                      email.}
	if { $has_eem_3_0 } {
            puts { --source             (optional) Source IP or interface to use}
            puts {                      when sending the email test.  If omitted}
            puts {                      the email will be sent from the interface}
            puts {                      closest to the SMTP server.}
	}
	if { $has_eem_2_4 } {
            puts { --username           (optional) SMTP server username.  If}
            puts {                      omitted, unauthenticated SMTP is}
            puts {                      assumed.}
            puts { --password           (optional) SMTP server password.  This}
            puts {                      can only be set if _email_username is}
            puts {                      also set.}
	}
    }
}

proc get_sch_counters { output fails successes } {
    set rfail 0
    set rsuccess 0
    upvar $fails num_bad_tests
    upvar $successes num_good_tests
    set num_good_tests 0
    set num_bad_tests 0

    foreach line [split $output "\n"] {
        set line [string trim $line]
        if { [regexp {^Total Success} $line] } {
	    set rfail 0
	    set rsuccess 1
	    continue
	}
	if { $rsuccess && [regexp {^Test\s+(\d+)} $line -> num_good_tests] } {
	    set rsuccess 0
	    continue
	}

	if { [regexp {^Total Failed} $line] } {
	    set rfail 1
	    set rsuccess 0
	    continue
	}

	if { $rfail && [regexp {^Test\s+(\d+)} $line -> num_bad_tests] } {
	    set rfail 0
	    continue
	}
    }
}

proc check_addrs { addrs } {
    set addrlst [split $addrs ","]
    foreach addr $addrlst {
	set addr [string trim $addr]
	if { ! [regexp {.+@.+} $addr] } {
	    puts "ERROR: $addr is not a valid email address."
	    return 0
	}
    }

    return 1
}

proc check_syslog { facility severities } {
    global syslog_facility
    global syslog_severities
    global syslog_hosts
    global LEVEL_TO_INT
    global DEF_SYSLOG_FACILITY

    set output [run_cli [list "show logging | begin Trap logging:"]]
    if { ! [regexp {level ([^,]+),} $output -> level] } {
	puts "ERROR: Syslog logging is not enabled."
	return 0
    }

    set trapsev $LEVEL_TO_INT($level)

    set willsend 0
    foreach severity $severities {
	if { ! [string is integer $severity] || $severity < 0 || $severity > 7 } {
	    puts "WARNING: Specified severity $severity is not a valid syslog severity."
	    puts "WARNING: Severities must be integers between 0 and 7."
        } elseif { $trapsev >= $severity } {
	    set willsend 1
	    lappend syslog_severities $severity
	} else {
	    puts "WARNING: The configured syslog trap level $level ($trapsev) will not allow"
	    puts "WARNING: a message with severity $severity to be sent."
	    puts "WARNING: To allow a message with severity $severity to be sent, configure:"
	    puts " logging trap $severity"
	}
    }

    if { ! $willsend } {
	puts "ERROR: No messages will be sent given the current syslog trap configuration."
	return 0
    }

    set tmphost ""
    set syslog_hosts [list]
    set found_host 0
    foreach line [split $output "\n"] {
	if { [regexp {Logging to ([\d\.]+)\s} $line -> host] } {
	    if { $tmphost != "" } {
		lappend syslog_hosts $tmphost
	    }
	    set tmphost $host
	    set found_host 1
	}

	if { $found_host && [regexp {\s\sfiltering enabled} $line] } {
	    set tmphost ""
	    set found_host 0
	}
    }

    if { $tmphost != "" } {
	lappend syslog_hosts $tmphost
    }

    if { [llength $syslog_hosts] == 0 } {
	puts "ERROR: There are no syslog hosts configured which can receive EEM syslog messages."
	return 0
    }

    if { $facility != "" } {
        set syslog_facility $facility
    } else {
	set syslog_facility $DEF_SYSLOG_FACILITY
    }

    return 1
}

proc check_trap { } {
    global trap_hosts

    set trap_hosts [list]

    set output [run_cli [list "show running | inc ^snmp-server enable traps event-manager|^snmp-server host"]]

    if { ! [regexp -line {^snmp-server enable traps event-manager} $output] } {
	puts "ERROR: Embedded Event Manager traps are not enabled."
	puts "ERROR: You must configure the following first:"
	puts ""
	puts " snmp-server enable traps event-manager"
	return 0
    }

    foreach line [split $output "\n"] {
	if { ! [regexp -line {^snmp-server host} $line] } {
	    continue
	}

	set line [string trim $line]

	if { [regexp -line {^snmp-server host ([\d\.]+) [^\s]+$} $line -> host] } {
	    lappend trap_hosts $host
	} elseif { [regexp -line {^snmp-server host ([\d\.]+).*\mevent-manager\M} $line -> host] } {
	    lappend trap_hosts $host
	}
    }

    if { [llength $trap_hosts] == 0 } {
	puts "ERROR: There are no trap receivers configured which can receive EEM traps."
	puts "ERROR: At least one trap host must be configured.  For example:"
	puts ""
	puts " snmp-server host 10.1.1.1 traps public"
	return 0
    }

    return 1
}

proc check_email { to from server {cc ""} {src ""} {user ""} {pass ""} } {
    global EMAIL_BODY
    global EMAIL_SUBJECT
    global email_tmpl
    global email_subj
    global email_recipients
    global has_eem_2_4
    global has_eem_3_0

    set tmpl {}

    array set sinfo [sys_reqinfo_routername]

    if { $server == "" } {
	puts "ERROR: The mail server name has not been specified."
	return 0
    }

    if { $has_eem_2_4 } {
        if { $user == "" && $pass != "" } {
	    puts "ERROR: The mail server username is empty, but the password is not."
	    puts "ERROR: Specify a valid SMTP username."
	    return 0
        } elseif { $pass != "" } {
	    set server "${user}:${pass}@${server}"
        } elseif { $user != "" } {
	    set server "${user}@${server}"
        }
    } elseif { $user != "" } {
	puts "WARNING: SMTP username/password support was not added until EEM 2.4."
	puts "WARNING: This test may fail if the SMTP server rejects the request."
    }

    append tmpl "Mailservername: $server\n"

    if { $from == "" } {
	puts "ERROR: The From address cannot be empty."
	return 0
    }
    if { ! [check_addrs $from] } {
	return 0
    }
    append tmpl "From: $from\n"

    if { $to == "" } {
	puts "ERROR: The To address cannot be empty."
	return 0
    }
    if { ! [check_addrs $to] } {
	return 0
    }
    append tmpl "To: $to\n"

    if { $cc != "" } {
	if { ! [check_addrs $cc] } {
	    return 0
	}
    }
    append tmpl "Cc: $cc\n"

    if { $has_eem_3_0 && $src != "" } {
	if { [regexp {^[\d\.]$} $src] } {
	    append tmpl "Sourceaddr: $src\n"
	} else {
	    append tmpl "Sourceintf: $src\n"
	}
    } elseif { ! $has_eem_3_0 && $src != "" } {
	puts "WARNING: Support for specifying the source IP/interface was not added until EEM 3.0."
	puts "WARNING: This message will be sent from the interface closest to the SMTP server."
    }

    regsub {%%ROUTERNAME%%} $EMAIL_SUBJECT $sinfo(routername) email_subj
    append tmpl "Subject: $email_subj\n\n"
    append tmpl $EMAIL_BODY
    regsub -all {%%ROUTERNAME%%} $tmpl $sinfo(routername) tmpl

    set email_tmpl $tmpl
    set email_recipients [split "$to,$cc" ","]

    return 1
}

proc check_sch { profile } {
    if { $profile == "" } {
	puts "ERROR: Profile name must be specified to test Smart Call Home."
	return 0
    }
    if { [catch {set output [run_cli [list "show call-home profile $profile"]]} result] || [regexp -line {^% Invalid input detected} $output] } {
	puts "ERROR: Device does not support Smart Call Home."
	return 0
    }
    if { [regexp {profile does not exist} $output] } {
	puts "ERROR: The Smart Call Home profile $profile does not exist."
	return 0
    }

    if { [regexp {Profile status: INACTIVE} $output] } {
	puts "ERROR: The Smart Call Home profile $profile is inactive."
	puts "ERROR: Only active profiles can be tested."
	return 0
    }

    set output [run_cli [list "show call-home"]]
    if { [regexp {call home feature : disable} $output] } {
	puts "ERROR: The Call Home service is not enabled."
	puts "ERROR: Configure \"service call-home\" to enable the service."
	return 0
    }

    return 1
}

variable SYSLOG_MSG {This is a test syslog brought to you by the EASy NMS Tester}
variable TRAP_STRDATA {This is a test trap brought to you by the EASy NMS Tester}
variable EMAIL_BODY {This is a test email brought to you by the EASy NMS Tester.

Email was sent from %%ROUTERNAME%%.

--
Visit us online at http://www.cisco.com/go/easy/
}
variable EMAIL_SUBJECT {EASy NMS Tester [%%ROUTERNAME%%]}
variable TRAP_INTDATA1 3
variable TRAP_INTDATA2 42
variable DEF_SYSLOG_FACILITY {HA_EM}
variable INT_TO_PRIO [list {emerg} {alert} {crit} {err} {warning} {notice} \
		      {info} {debug}]
array set LEVEL_TO_INT [list {emergencies} 0 {alerts} 1 {critical} 2 \
		        {errors} 3 {warnings} 4 {notifications} 5 \
		        {informational} 6 {debugging} 7]

set tests [list]
set unsup_tests [list]
set policy_name {no_easy_nmstest.tcl}
set errors 0
set bad_args 0
set has_eem_2_4 0
set has_eem_3_0 0
set has_eem_3_1 0
set syslog_facility {}
set syslog_severities [list]
set syslog_hosts [list]

array set arr_einfo [event_reqinfo]
set argc 0
set arg {}
if { [info exists arr_einfo(argc)] } {
    set has_eem_2_4 1
    set argc $arr_einfo(argc)
    for { set i 1 } { $i <= $argc } { incr i } {
	append arg "$arr_einfo(arg$i) "
    }
}

set arg [string trim $arg]

if { ! [catch {run_cli [list "show event manager version"]} result] } {
    if { [regexp {Embedded Event Manager Version (\d+\.\d)} $result -> vers] } {
	if { $vers >= 3.1 } {
	    set has_eem_3_1 1
	}
	if { $vers >= 3.0 } {
	    set has_eem_3_0 1
	}
    }
}

if { $argc > 0 } {
    set tmptests [list]
    set facility {}
    set severities [list 0 1 2 3 4 5 6]
    set to {}
    set from {}
    set cc {}
    set server {}
    set src {}
    set profile {}
    set username {}
    set password {}
    set args [split $arg]

    for { set i 0 } { $i < [llength $args] } { incr i } {
	switch -glob -- [lindex $args $i] {
	    --tests { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set tmptests [split $result ","]
		}
	    }
	    --facility { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set facility $result
		}
	    }
	    --severities { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set severities [split $result ","]
		}
	    }
	    --to { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set to $result
		}
	    }
	    --from { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set from $result
		}
	    }
	    --cc { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set cc $result
		}
	    }
	    --server { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set server $result
		}
	    }
	    --source { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set src $result
		}
	    }
	    --username { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set username $result
		}
	    }
	    --password { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set password $result
		}
	    }
	    --profile { if { [catch {lindex $args [incr i]} result] } {
		    usage
		    exit 1
		} else {
		    set profile $result
		}
	    }
	    --help { cmdhelp
		exit 0
	    }
	    -* { puts "Unknown option, [lindex $args $i]"
		 puts "Run '$policy_name --help' to see a full list of available command line options."
		 exit 1
	     }
	     default { usage
		 exit 1
	     }
	}
    }

    foreach tmptest $tmptests {
	if { $tmptest == "syslog" } {
	    if { [check_syslog $facility $severities] } {
		lappend tests "syslog"
	    } else {
		lappend unsup_tests "syslog"
		incr errors
	    }
	} elseif { $tmptest == "trap" } {
	    if { [check_trap] } {
		lappend tests "trap"
	    } else {
		lappend unsup_tests "trap"
		incr errors
	    }
	} elseif { $tmptest == "email" } {
	    if { [check_email $to $from $server $cc $src $username $password] } {
		lappend tests "email"
	    } else {
		lappend unsup_tests "email"
		incr errors
	    }
	} elseif { $tmptest == "callhome" } {
	    if { [check_sch $profile] } {
		lappend tests "callhome"
	    } else {
		lappend unsup_tests "callhome"
		incr errors
	    }
	} else {
	    puts "WARNING: Invalid test name '$tmptest'."
	    incr errors
	    incr bad_args
	}
    }
} else {
    if { [info exists easy_nmstest_syslog] && $easy_nmstest_syslog != 0 } {
        set facility $DEF_SYSLOG_FACILITY
        set severities [list 0 1 2 3 4 5 6]
        if { [info exists easy_nmstest_syslog_fac] } {
	    set facility $easy_nmstest_syslog_fac
        }
        if { [info exists easy_nmstest_syslog_sevs] } {
	    set severities [split $easy_nmstest_syslog_sevs ","]
        }
        if { [check_syslog $facility $severities] } {
	    lappend tests "syslog"
        } else {
	    lappend unsup_tests "syslog"
	    incr errors
        }
    }

    if { [info exists easy_nmstest_trap] && $easy_nmstest_trap != 0 } {
        if { [check_trap] } {
	    lappend tests "trap"
        } else {
	    lappend unsup_tests "trap"
	    incr errors
        }
    }

    if { [info exists easy_nmstest_email] && $easy_nmstest_email != 0 } {
	if { ! [info exists easy_nmstest_email_server] } {
            if { ! [info exists _email_server] } {
	        set easy_nmstest_email_server ""
	    } else {
		set easy_nmstest_email_server $_email_server
	    }
        }

	if { ! [info exists easy_nmstest_email_from] } {
            if { ! [info exists _email_from] } {
	        set easy_nmstest_email_from ""
	    } else {
		set easy_nmstest_email_from $_email_from
	    }
        }

	if { ! [info exists easy_nmstest_email_to] } {
            if { ! [info exists _email_to] } {
	        set easy_nmstest_email_to ""
	    } else {
		set easy_nmstest_email_to $_email_to
	    }
        }

	if { ! [info exists easy_nmstest_email_cc] } {
            if { ! [info exists _email_cc] } {
	        set easy_nmstest_email_cc ""
	    } else {
		set easy_nmstest_email_cc $_email_cc
	    }
        }

        set src ""
	if { [info exists easy_nmstest_email_source_addr] } {
	    set src $easy_nmstest_source_addr
	} elseif { [info exists easy_nmstest_email_source_intf] } {
	    set src $easy_nmstest_source_intf
        } elseif { [info exists _email_source_addr] } {
	    set src $_email_source_addr
        } elseif { [info exists _email_source_intf] } {
	    set src $_email_source_intf
        }

        set user ""
        set pass ""
	if { [info exists easy_nmstest_email_user] } {
	    set user $easy_nmstest_email_user
	} elseif { [info exists _email_user] } {
	    set user $_email_user
	}
	if { [info exists easy_nmstest_email_pass] } {
	    set pass $easy_nmstest_email_pass
        } elseif { [info exists _email_pass] } {
	    set pass $_email_pass
        }

        if { [check_email $easy_nmstest_email_to $easy_nmstest_email_from $easy_nmstest_email_server $easy_nmstest_email_cc $src $user $pass] } {
	    lappend tests "email"
        } else {
	    lappend unsup_tests "email"
	    incr errors
        }
    }

    if { [info exists easy_nmstest_callhome] && $easy_nmstest_callhome != 0 } {
	set profile ""
	if { [info exists easy_nmstest_callhome_profile] } {
	    set profile $easy_nmstest_callhome_profile
	}
        if { [check_sch $profile] } {
	    lappend tests "callhome"
        } else {
	    lappend unsup_tests "callhome"
	    incr errors
        }
    }
}

if { [llength $tests] == 0 } {
    puts "\nERROR: Did not find any tests to run."
    if { $bad_args } {
        puts "ERROR: Check your configuration and command line arguments."
        usage
	exit 1
    }
    if { $errors } {
	puts "ERROR: Some tests could not be run due to errors:"
	foreach t $unsup_tests {
	    puts "\t$t"
	}
    }
    exit 1
}

puts ""

foreach test $tests {
    if { $test == "syslog" } {
	puts "INFO: Starting syslog test..."
	puts ""
	foreach severity $syslog_severities {
	    if { [catch {open "syslog:" "w"} result] } {
	        if { $has_eem_3_1 } {
		    action_syslog priority [lindex $INT_TO_PRIO $severity] facility $syslog_facility msg $SYSLOG_MSG
		} else {
		    if { $syslog_facility != $DEF_SYSLOG_FACILITY } {
			puts "WARNING: Cannot send a message with $syslog_facility as this device does not support EEM 3.1 or the 'syslog:' file system."
		        set syslog_facility $DEF_SYSLOG_FACILITY
		    }
		    action_syslog priority [lindex $INT_TO_PRIO $severity] msg $SYSLOG_MSG
		}
	    } else {
		set fd $result
		puts -nonewline $fd "%${syslog_facility}-$severity-LOG: $policy_name: $SYSLOG_MSG"
		flush $fd
		close $fd
	    }

	    puts "INFO: The following syslog message was just sent:"
	    puts ""
	    puts " %${syslog_facility}-$severity-LOG: $policy_name: $SYSLOG_MSG"
	    puts ""
	    puts "INFO: The following syslog hosts should have received it:"
	    puts ""
	    foreach host $syslog_hosts {
		puts " $host"
	    }
	    puts ""
	}

	puts ""
	puts "INFO: Syslog test complete."
	puts ""
    }

    if { $test == "trap" } {
	puts "INFO: Starting SNMP trap test..."
	puts ""
	action_snmp_trap intdata1 $TRAP_INTDATA1 intdata2 $TRAP_INTDATA2 strdata $TRAP_STRDATA
	puts "INFO: A cEventMgrPolicyEvent (1.3.6.1.4.1.9.10.91.0.2) trap was sent with"
	puts "INFO: the following varbinds:"
	puts ""
	puts " ceemHistoryPolicyIntData1 (1.3.6.1.4.1.9.10.91.1.2.3.1.9)  : $TRAP_INTDATA1"
	puts " ceemHistoryPolicyIntData2 (1.3.6.1.4.1.9.10.91.1.2.3.1.10) : $TRAP_INTDATA2"
	puts " ceemHistoryPolicyStrData  (1.3.6.1.4.1.9.10.91.1.2.3.1.11) : $TRAP_STRDATA"
	puts ""
	puts "INFO: The trap was sent to the following trap receiver(s):"
	puts ""
	foreach host $trap_hosts {
	    puts " $host"
	}
	puts ""
	puts "INFO: SNMP trap test complete."
	puts ""
    }

    if { $test == "email" } {
	puts "INFO: Starting email test..."
	puts ""
	if { [catch {smtp_send_email $email_tmpl} result] } {
	    puts "ERROR: Failed to send email: '$result'"
	    incr errors
	} else {
	    puts "INFO: Email was successfully sent to the following recipients:"
	    puts ""
	    foreach recip $email_recipients {
		puts " $recip"
	    }
	    puts ""
	    puts "INFO: The email will appear with subject: '$email_subj'"
	}
	puts ""
	puts "INFO: Email test complete."
	puts ""
    }

    if { $test == "callhome" } {
	puts "INFO: Starting Smart Call Home test..."
	puts ""
	set output [run_cli [list "show call-home statistics"]]
	get_sch_counters $output pre_fails pre_successes
	if { [catch {run_cli [list "call-home test profile $profile"]} result] } {
	    puts "ERROR: Failed to initiate Smart Call Home test: '$result'"
	    incr errors
	} else {
	    if { [regexp {Call Home needs the following.*\n(.*)} $result -> sch_errors] } {
		puts "ERROR: Smart Call Home test failed due to the following error(s):"
		set elst [split $sch_errors "\n"]
		foreach e $elst {
		    puts "ERROR:\t$e"
		}
		incr errors
	    }

	    after 7500

	    set output [run_cli [list "show call-home statistics"]]
	    get_sch_counters $output post_fails post_successes
	    if { $post_fails > $pre_fails } {
		puts "ERROR: There was a failure in generating the Smart Call Home Test."
		incr errors
	    } elseif { $post_successes > $pre_successes } {
		puts "INFO: Smart Call Home test completed successfully."
	    } else {
		puts "WARNING: Unable to determine if the Smart Call Home test was successful or not."
	    }
	}

	puts ""
	puts "INFO: Smart Call Home test complete."
	puts ""
    }
}

if { $errors } {
    puts "ERROR: Test(s) completed with errors."
    if { [llength $unsup_tests] > 0 } {
	puts "ERROR: Some tests could not be attempted due to errors:"
	foreach t $unsup_tests {
	    puts "\t$t"
	}
    }
    puts "ERROR: See the output above for more details."
    exit 1
}
